@@ -81,7 +81,7 @@ void pathdefaults(pathptr newpath, int x, int y)
 /* Sane values for a new object instance */
 /*---------------------------------------*/
 
-void objectdefaults(objinstptr newinst, objectptr thisobj, int x, int y)
+void instancedefaults(objinstptr newinst, objectptr thisobj, int x, int y)
 {
    newinst->position.x = x;
    newinst->position.y = y;
@@ -115,7 +115,8 @@ void drawdot(int xpos, int ypos)
    if ((dotobj = finddot()) != (objectptr)NULL) {
       NEW_OBJINST(newdot, topobject);
       topobject->parts++;
-      objectdefaults(*newdot, dotobj, xpos, ypos);
+      instancedefaults(*newdot, dotobj, xpos, ypos);
+      register_for_undo(XCF_Dot, UNDO_DONE, areastruct.topinstance, *newdot);
    }
    else {
       NEW_ARC(newarc, topobject);
@@ -127,6 +128,7 @@ void drawdot(int xpos, int ypos)
       (*newarc)->style = FILLED | FILLSOLID | NOBORDER;
       (*newarc)->passed = NULL;
       calcarc(*newarc);
+      register_for_undo(XCF_Arc, UNDO_DONE, areastruct.topinstance, *newarc);
    }
    incr_changes(topobject);
 }
@@ -254,7 +256,7 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
    labelptr tlab;
 
    for (tgen = topobject->plist; tgen < topobject->plist + topobject->parts; tgen++) {
-      if ((*tgen)->type == LABEL) {
+      if (IS_LABEL(*tgen)) {
          tlab = TOLABEL(tgen);
 	 if (tlab->pin == False) continue;
 	 else if (tlab == curlabel) continue;  /* Don't count self! */
@@ -367,7 +369,7 @@ void labeltext(int keypressed, char *clientdata)
 
 	 areastruct.save.y = curlabel->position.y + 100;
 	 areastruct.save.x = curlabel->position.x;
-	 objectselect(OBJECT);
+	 select_element(OBJINST);
 	 if (areastruct.selects == 1) {  /* this should be true! */
 	    short *selptr = areastruct.selectlist + areastruct.selects - 1;
 	    objinstptr selobj = SELTOOBJINST(selptr);
@@ -387,7 +389,7 @@ void labeltext(int keypressed, char *clientdata)
 	    }
 	 }
          eventmode = CATALOG_MODE;
-	 objectdeselect();
+	 unselect_all();
       }
       else {	/* (hasstuff && eventmode != CATTEXT_MODE) */
 	 eventmode = NORMAL_MODE;
@@ -682,7 +684,7 @@ void rejustify(short mode)
 	    redrawtext(curlabel);
 	 }
       }
-      if (eventmode != PRESS_MODE && eventmode != COPY2_MODE) objectdeselect();
+      if (eventmode != PRESS_MODE && eventmode != COPY2_MODE) unselect_all();
    }
    if (curlabel == NULL)
       Wprintf("No labels chosen to rejustify");
@@ -1204,7 +1206,7 @@ void setendpoint(short *scnt, short direc, XPoint **endpoint, XPoint *arcpoint)
 {
    genericptr *sptr = topobject->plist + (*scnt);
 
-   switch((*sptr)->type) {
+   switch(ELEMENTTYPE(*sptr)) {
       case POLYGON:
 	 if (direc)
 	    *endpoint = TOPOLY(sptr)->points + TOPOLY(sptr)->number - 1;
@@ -1281,7 +1283,7 @@ void reversefpoints(XfPoint *plist, short number)
 void freeparts(short *selectobj, short add)
 {
    genericptr *oldelem = topobject->plist + (*selectobj);
-   switch((*oldelem)->type) {
+   switch(ELEMENTTYPE(*oldelem)) {
       case POLYGON:
 	 free((TOPOLY(oldelem))->points);
 	 break;
@@ -1321,7 +1323,7 @@ void unjoin()
    genericptr *genp, *newg;
    pathptr oldpath;
 
-   if (areastruct.selects == 0) objectselect(PATH);
+   if (areastruct.selects == 0) select_element(PATH);
    if (areastruct.selects == 0) {
       Wprintf("No objects selected.");
       return;
@@ -1610,7 +1612,7 @@ void join()
 	 /* reverse point order if necessary */
 
          if (*(direc + (short)(sptr - order)) == 1) {
-	    switch ((*newelem)->type) {
+	    switch (ELEMENTTYPE(*newelem)) {
 	       case POLYGON:
 		  reversepoints(TOPOLY(newelem)->points, TOPOLY(newelem)->number);
 	          break;
@@ -1715,6 +1717,7 @@ void wirebutton(XButtonEvent *event)
 	 topobject->parts++;
 	 incr_changes(topobject);
 	 if (!nonnetwork(newwire)) invalidate_netlist(topobject);
+	 register_for_undo(XCF_Wire, UNDO_DONE, areastruct.topinstance, newwire);
       }
       UDrawPolygon(newwire);
       if (event->button == Button3)
